import {AfterViewInit, Component, ElementRef, OnInit, ViewChild} from '@angular/core';
import PhylocanvasGL, {TreeTypes} from "@phylocanvas/phylocanvas.gl";
import {Router} from '@angular/router';
import {DataService} from 'src/app/services/data.service';
import {MatDialog} from '@angular/material/dialog';
import {DialogComponent} from '../dialog/dialog.component';
import {SLFDatabase} from "../../models/SLFDatabase";
import {MatSnackBar} from "@angular/material/snack-bar";

@Component({
  selector: 'slf-home',
  templateUrl: './home.component.html',
  styleUrls: ['./home.component.scss']
})
export class HomeComponent implements OnInit, AfterViewInit {
  @ViewChild('demo') demoDiv!: ElementRef<HTMLDivElement>;
  private tree: PhylocanvasGL;
  private SLFDatabase?: SLFDatabase;

  constructor(private dataService: DataService,
              private router: Router,
              public dialog: MatDialog, private readonly snackBar: MatSnackBar) {
  }

  ngOnInit(): void {

  }

  ngAfterViewInit() {
    this.dataService.getDatabase()
      .subscribe(slfd => {
        this.SLFDatabase = slfd;
        this.renderizeFilogeneticTree();
      });
  }

  private renderizeFilogeneticTree(): void {
    const screenSize = this.demoDiv.nativeElement.getBoundingClientRect();
    screenSize.width = screenSize.width * 0.95;
    screenSize.height = screenSize.height * 3;
    const metadata = 'id,colour\nRosaceae,red\nUlmaceae,red'; //Esto serian diferentes opciones de personalización de nodos en formato csv
    this.tree = new PhylocanvasGL(
      this.demoDiv.nativeElement,
      {
        interactive: false,
        showLabels: true,
        showLeafLabels: true,
        size: screenSize,
        alignLabels: true,
        nodeSize: 16,
        fillColour: '#3f51b5',
        source: this.SLFDatabase?.newick,
        type: TreeTypes.Rectangular,
        styles: (
          metadata
            .split("\n")
            .map((line) => line.split(','))
            .reduce(
              (styles, [id, fillColour]) => {
                styles[id] = {fillColour};
                return styles;
              },
              {}
            )
        ),
      },
    );
    this.tree.setProps({
      interactive: true
    });
  }

  public onGoToFamily(): void {
    const param = this.tree['props']['selectedIds']
    if ((param !== undefined) && (this.SLFDatabase !== undefined) && (String(param) !== "")) {


      if (param == 'AmaranthaceaeChenopodiaceae') {
        let existAmaranthaceae = "Amaranthaceae" in this.SLFDatabase?.families
        let existChenopodiaceae = "Chenopodiaceae" in this.SLFDatabase?.families
        this.onOpenDialog(existAmaranthaceae, existChenopodiaceae);
      } else {
        if (param in this.SLFDatabase?.families) {
          this.router.navigate(['family/' + param]);
        } else { //Si no muestro error


          this.snackBar.open(`Data not found for family: ${param}`, 'Close', {
            horizontalPosition: "end",
            verticalPosition: "bottom",
            duration: 4000
          });


        }
      }

    }

  }

  onOpenDialog(existAmaranthaceae: boolean, existChenopodiaceae: boolean): void {
    const dialogRef = this.dialog.open(DialogComponent, {
      width: '450px',
      data: {'existAmaranthaceae': existAmaranthaceae, 'existChenopodiaceae': existChenopodiaceae}
    });
  }

}
